#include "ArticulatedObjectProviderPlugin.h"

namespace armarx::plugins
{

    void ArticulatedObjectProviderPlugin::postCreatePropertyDefinitions(PropertyDefinitionsPtr& properties)
    {
        if (!properties->hasDefinition(makePropertyName(propertyName)))
        {
            properties->defineOptionalProperty<std::string>(
                makePropertyName(propertyName),
                "ArticulatedObjectTopic",
                "Name of the articulated object topic.");
        }
    }

    void ArticulatedObjectProviderPlugin::preOnInitComponent()
    {
        parent<Component>().offeringTopicFromProperty(makePropertyName(propertyName));
    }

    void ArticulatedObjectProviderPlugin::preOnConnectComponent()
    {
        parent<articulated_object::ArticulatedObjectProviderPluginUser>().articulatedObjectTopic = createArticulatedObjectTopic();
    }

    void ArticulatedObjectProviderPlugin::postOnConnectComponent()
    {
        // TODO(fabian.reister): implement

        // ArticulatedObjectProviderPluginUser& parent = this->parent<ArticulatedObjectProviderPluginUser>();
        // objpose::ObjectPoseTopicPrx& topic = parent.articulatedObjectTopic;
        // if (topic)
        // {
        //     objpose::ProviderInfo info = parent.getProviderInfo();
        //     if (!info.proxy)
        //     {
        //         info.proxy = ::armarx::objpose::ObjectPoseProviderPrx::checkedCast(parent.getProxy());
        //     }
        //     topic->reportProviderAvailable(parent.getName(), info);
        // }
        // else
        // {
        //     ARMARX_ERROR << "Object pose topic not ready.";
        // }
    }

    articulated_object::ArticulatedObjectTopicPrx ArticulatedObjectProviderPlugin::createArticulatedObjectTopic()
    {
        return parent<Component>().getTopicFromProperty<articulated_object::ArticulatedObjectTopicPrx>(makePropertyName(propertyName));
    }

}  // namespace armarx::plugins


namespace armarx::articulated_object
{

    ArticulatedObjectProviderPluginUser::ArticulatedObjectProviderPluginUser()
    {
        addPlugin(plugin);
    }

    provider::TrackingRequestResponse ArticulatedObjectProviderPluginUser::trackObjects(const provider::TrackingRequest& input, const ::Ice::Current&)
    {
        ARMARX_INFO << "Requested " << input.objectIDs.size() << " objects for " << input.relativeTimeoutMS << " ms.";
        provider::TrackingRequestResponse output;
        for (const auto& id : input.objectIDs)
        {
            output.status[id] = false;
        }
        return output;
    }

    ArticulatedObjectTopicPrx ArticulatedObjectProviderPluginUser::createArticulatedObjectTopic()
    {
        return plugin->createArticulatedObjectTopic();
    }

}  // namespace armarx::articulated_object
