/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once


// TODO(fabian.reister): remove
#include <ArmarXCore/core/application/properties/PropertyDefinitionContainer.h>

#include <RobotAPI/interface/armem/mns/MemoryNameSystemInterface.h>

namespace armarx
{
    class ManagedIceObject;
}

namespace armarx::armem
{

    /**
     * @brief The MemoryConnector class simplifies connecting to the ArMem memory.
     *
     * Use this as the base class of any class that needs to connect to the memory.
     *
     *
     */
    class MemoryConnector
    {

    public:
        MemoryConnector(ManagedIceObject& component);
        virtual ~MemoryConnector();

    protected:
        armem::data::WaitForMemoryResult useMemory(const std::string& memoryName);
        void registerPropertyDefinitions(PropertyDefinitionsPtr& def);

        void waitForMemory();

        virtual const std::string& getPropertyPrefix() const;

    private:
        ManagedIceObject& component;

        armem::mns::MemoryNameSystemInterfacePrx memoryNameSystem;

        const std::string propertyPrefix;
    };

} // namespace armarx::armem