#pragma once

#include "../base/EntityBase.h"

#include "EntitySnapshot.h"

#include "../workingmemory/Entity.h"


namespace armarx::armem::ltm
{
    /**
     * @brief An entity over a period of time.
     *
     * An entity should be a physical thing or abstract concept existing
     * (and potentially evolving) over some time.
     *
     * Examples are:
     * - objects (the green box)
     * - agents (robot, human)
     * - locations (frige, sink)
     * - grasp affordances (general, or for a specific object)
     * - images
     * - point clouds
     * - other sensory values
     *
     * At each point in time (`EntitySnapshot`), the entity can have a
     * (potentially variable) number of instances (`EntityInstance`),
     * each containing a single `AronData` object of a specific `AronType`.
     */
    class Entity :
        public base::EntityBase<EntitySnapshot, Entity>
    {
        using Base = base::EntityBase<EntitySnapshot, Entity>;

    public:

        using Base::EntityBase;
        using Base::operator=;


        // Conversion
        wm::Entity convert() const;

        // MongoDB connection
        void append(const wm::Entity&);
    };

}
