#include "ice_conversions.h"

namespace armarx::armem
{
    void toIce(armarx::armem::data::EntityInstanceMetadata& ice, const armarx::armem::wm::EntityInstanceMetadata& metadata)
    {
        ice.confidence = metadata.confidence;
        toIce(ice.timeArrivedMicroSeconds, metadata.timeArrived);
        toIce(ice.timeCreatedMicroSeconds, metadata.timeCreated);
        toIce(ice.timeSentMicroSeconds, metadata.timeSent);
    }
    void fromIce(const armarx::armem::data::EntityInstanceMetadata& ice, armarx::armem::wm::EntityInstanceMetadata& metadata)
    {
        metadata.confidence = ice.confidence;
        fromIce(ice.timeArrivedMicroSeconds, metadata.timeArrived);
        fromIce(ice.timeCreatedMicroSeconds, metadata.timeCreated);
        fromIce(ice.timeSentMicroSeconds, metadata.timeSent);
    }

    void toIce(data::EntityInstance& ice, const wm::EntityInstance& data)
    {
        detail::toIceItem(ice, data);

        if (data.data())
        {
            ice.data = data.data()->toAronDictPtr();
        }
        toIce(ice.metadata, data.metadata());
    }
    void fromIce(const data::EntityInstance& ice, wm::EntityInstance& data)
    {
        detail::fromIceItem(ice, data);

        if (ice.data)
        {
            data.setData(aron::datanavigator::DictNavigator::FromAronDictPtr(ice.data));
        }
        fromIce(ice.metadata, data.metadata());
    }


    void toIce(data::EntitySnapshot& ice, const wm::EntitySnapshot& snapshot)
    {
        detail::toIceItem(ice, snapshot);

        toIce(ice.instances, snapshot.instances());
    }
    void fromIce(const data::EntitySnapshot& ice, wm::EntitySnapshot& snapshot)
    {
        detail::fromIceItem(ice, snapshot);

        fromIce(ice.instances, snapshot.instances());
    }

    void toIce(data::Entity& ice, const wm::Entity& entity)
    {
        detail::toIceItem(ice, entity);

        toIce(ice.history, entity.history());
    }
    void fromIce(const data::Entity& ice, wm::Entity& entity)
    {
        detail::fromIceItem(ice, entity);

        fromIce(ice.history, entity.history());
    }


    void toIce(data::ProviderSegment& ice, const wm::ProviderSegment& providerSegment)
    {
        detail::toIceItem(ice, providerSegment);

        if (providerSegment.hasAronType())
        {
            ice.aronType = providerSegment.aronType()->getResult();
        }
        toIce(ice.entities, providerSegment.entities());
    }
    void fromIce(const data::ProviderSegment& ice, wm::ProviderSegment& providerSegment)
    {
        detail::fromIceItem(ice, providerSegment);

        if (ice.aronType)
        {
            providerSegment.aronType() = aron::typenavigator::ObjectNavigator::DynamicCastAndCheck(aron::typenavigator::Navigator::FromAronType(ice.aronType));
        }
        fromIce(ice.entities, providerSegment.entities());
    }

    void toIce(data::CoreSegment& ice, const wm::CoreSegment& coreSegment)
    {
        detail::toIceItem(ice, coreSegment);

        if (coreSegment.hasAronType())
        {
            ice.aronType = coreSegment.aronType()->getResult();
        }
        toIce(ice.providerSegments, coreSegment.providerSegments());
    }
    void fromIce(const data::CoreSegment& ice, wm::CoreSegment& coreSegment)
    {
        detail::fromIceItem(ice, coreSegment);

        if (ice.aronType)
        {
            coreSegment.aronType() = aron::typenavigator::ObjectNavigator::DynamicCastAndCheck(aron::typenavigator::Navigator::FromAronType(ice.aronType));
        }
        fromIce(ice.providerSegments, coreSegment.providerSegments());
    }

    void toIce(data::Memory& ice, const wm::Memory& memory)
    {
        detail::toIceItem(ice, memory);

        toIce(ice.coreSegments, memory.coreSegments());
    }
    void fromIce(const data::Memory& ice, wm::Memory& memory)
    {
        detail::fromIceItem(ice, memory);

        fromIce(ice.coreSegments, memory.coreSegments());
    }

}
