#pragma once

#include "BaseQueryProcessor.h"
#include "../base/MemoryQueryProcessorBase.h"

#include "CoreSegmentQueryProcessor.h"

namespace armarx::armem::wm::query_proc
{
    /**
     * @brief Handles memory queries.
     */
    class MemoryQueryProcessor :
        virtual public BaseQueryProcessor<wm::Memory, armem::query::data::MemoryQuery>,
        virtual public base::query_proc::MemoryQueryProcessorBase<wm::Memory>
    {
        using Base = BaseQueryProcessor<wm::Memory, armem::query::data::MemoryQuery>;

    public:
        MemoryQueryProcessor(DataMode dataMode = DataMode::WithData) :
            Base(dataMode), coreSegmentProcessor(dataMode)
        {}

        armem::query::data::Result process(const armem::query::data::Input& input,
                                           const wm::Memory& memory) const
        {
            armem::query::data::Result result;
            result.memory = processToIce(input.memoryQueries, memory);
            result.success = true;
            return result;
        }

        using Base::process;
        data::MemoryPtr processToIce(const armem::query::data::MemoryQuery& query,
                                     const wm::Memory& memory) const
        {
            return toIce<data::MemoryPtr>(process(query, memory));
        }

        data::MemoryPtr processToIce(const armem::query::data::MemoryQuerySeq& queries,
                                     const wm::Memory& memory) const
        {
            return toIce<data::MemoryPtr>(process(queries, memory));
        }

    protected:
        virtual CoreSegmentT coreSegmentProcessorProcess(const armem::query::data::CoreSegmentQuerySeq& q, const CoreSegmentT& s) const override
        {
            return coreSegmentProcessor.process(q, s);
        }


    private:
        CoreSegmentQueryProcessor coreSegmentProcessor;
    };
}
