#pragma once

#include <optional>

#include <QWidget>

#include <RobotAPI/libraries/aron/core/navigator/type/container/Object.h>

#include <RobotAPI/libraries/armem/core/workingmemory/Memory.h>


class QGroupBox;
class QLabel;
class QSplitter;
class QTreeWidget;
class QTreeWidgetItem;


namespace armarx::armem::gui::instance
{
    class ImageView;


    class InstanceView : public QWidget
    {
        Q_OBJECT
        using This = InstanceView;


    public:

        InstanceView();

        void setStatusLabel(QLabel* statusLabel);
        void setUseTypeInfo(bool enable);

        void update(const MemoryID& id, const wm::Memory& memory);
        void update(const wm::EntityInstance& instance, aron::typenavigator::ObjectNavigatorPtr aronType = nullptr);
        void update();


    signals:

        void updated();
        void instanceSelected(const MemoryID& id);


    private slots:

        void prepareTreeContextMenu(const QPoint& pos);
        void showImageView(const aron::Path& elementPath);
        void removeImageView();


    private:

        void updateInstanceID(const MemoryID& id);
        void updateData(const aron::datanavigator::DictNavigatorPtr& data, aron::typenavigator::ObjectNavigatorPtr aronType = nullptr);
        void updateMetaData(const wm::EntityInstanceMetadata& metadata);
        void updateImageView(const aron::datanavigator::DictNavigatorPtr& data);

        void showErrorMessage(const std::string& message);


    private:

        enum class Columns
        {
            KEY = 0,
            VALUE = 1,
            TYPE = 2,
        };

        std::optional<wm::EntityInstance> currentInstance;
        aron::typenavigator::ObjectNavigatorPtr currentAronType = nullptr;
        bool useTypeInfo = true;

        QSplitter* splitter;

        QTreeWidget* tree;
        QTreeWidgetItem* treeItemInstanceID;
        QTreeWidgetItem* treeItemMetadata;
        QTreeWidgetItem* treeItemData;


        class ImageView
        {
        public:
            ImageView(QSplitter* parent);

            QGroupBox* group;
            instance::ImageView* view;
            aron::Path elementPath;
        };
        std::optional<ImageView> imageView;

        QLabel* statusLabel = nullptr;

    };

}

namespace armarx::armem::gui
{
    using InstanceView = instance::InstanceView;
}
