#pragma once

#include <mutex>
#include <optional>

#include "RobotAPI/libraries/armem/client/MemoryConnector.h"
#include "RobotAPI/libraries/armem/client/Reader.h"

#include "interfaces.h"

namespace armarx::armem::articulated_object
{
    class Reader:
        virtual public ReaderInterface,
        virtual public ::armarx::armem::MemoryConnector
    {
    public:
        Reader(ManagedIceObject& component);
        virtual ~Reader() = default;

        void synchronize(ArticulatedObject& obj, const armem::Time& timestamp) override;

        std::optional<ArticulatedObject> get(const std::string& name, const armem::Time& timestamp) override;
        ArticulatedObject get(const ArticulatedObjectDescription& description, const armem::Time& timestamp) override;

        std::optional<RobotState> queryState(const RobotDescription& description, const armem::Time& timestamp);
        std::optional<RobotDescription> queryDescription(const std::string& name, const armem::Time& timestamp);


        // TODO(fabian.reister): register property defs

    protected:
        std::optional<RobotState> getRobotState(const armarx::armem::wm::Memory& memory) const;
        std::optional<RobotDescription> getRobotDescription(const armarx::armem::wm::Memory& memory) const;

    private:

        struct Properties
        {
            std::string memoryName               = "Object";
            std::string coreInstanceSegmentName  = "ArticulatedObjectInstance";
            std::string coreClassSegmentName     = "ArticulatedObjectClass";
            std::string providerName;
        } properties;

        const std::string propertyPrefix = "mem.obj.articulated.";

        armem::client::Reader memoryReader;
        std::mutex memoryWriterMutex;

    };


}  // namespace armarx::armem::articulated_object