#pragma once

#include <mutex>

#include "RobotAPI/libraries/armem/client/MemoryConnector.h"
#include "RobotAPI/libraries/armem/client/Reader.h"
#include "RobotAPI/libraries/armem/client/Writer.h"

#include "interfaces.h"


namespace armarx::armem::articulated_object
{

    class Writer:
        virtual public WriterInterface,
        virtual public MemoryConnector
    {
    public:
        Writer(ManagedIceObject& component);
        virtual ~Writer() = default;


        void registerPropertyDefinitions(armarx::PropertyDefinitionsPtr& def);
        void connect();


        bool store(const ArticulatedObject& obj) override;

        bool storeInstance(const ArticulatedObject& obj);
        std::optional<armem::MemoryID> storeClass(const ArticulatedObject& obj);

        const std::string& getPropertyPrefix() const override;

    private:
        std::optional<armem::MemoryID> storeOrGetClass(const ArticulatedObject& obj);

        void updateKnownObjects(const armem::MemoryID& subscriptionID, const std::vector<armem::MemoryID>& snapshotIDs);
        void updateKnownObject(const armem::MemoryID& snapshotId);

        struct Properties
        {
            std::string memoryName              = "Object";
            std::string coreInstanceSegmentName = "ArticulatedObjectInstance";
            std::string coreClassSegmentName    = "ArticulatedObjectClass";
            std::string providerName;
        } properties;

        const std::string propertyPrefix = "mem.obj.articulated.";

        armem::client::Writer memoryWriter;
        std::mutex memoryWriterMutex;

        armem::client::Reader memoryReader;
        std::mutex memoryReaderMutex;

        // key: name of object: RobotDescription::name
        std::map<std::string, MemoryID> knownObjects;
    };


}  // namespace armarx::armem::articulated_object