#include "Segment.h"

#include <RobotAPI/libraries/armem_objects/aron_conversions.h>

#include <RobotAPI/libraries/armem/core/aron_conversions.h>
#include <RobotAPI/libraries/armem/core/workingmemory/Visitor.h>
#include <RobotAPI/libraries/armem/client/Writer.h>
#include <RobotAPI/libraries/armem/client/query/Builder.h>
#include <RobotAPI/libraries/armem/client/query/query_fns.h>

#include <RobotAPI/libraries/ArmarXObjects/ObjectFinder.h>
#include <RobotAPI/libraries/ArmarXObjects/aron_conversions.h>
#include <RobotAPI/libraries/ArmarXObjects/ice_conversions.h>
#include <RobotAPI/libraries/aron/common/aron/Robot.aron.generated.h>

#include <RobotAPI/libraries/core/FramedPose.h>
#include <RobotAPI/libraries/core/remoterobot/RemoteRobot.h>

#include <ArmarXCore/core/time/TimeUtil.h>

#include <sstream>


#include "RobotAPI/libraries/aron/common/aron_conversions.h"

namespace armarx::armem::server::obj::articulated_object
{

    Segment::Segment(armem::server::MemoryToIceAdapter& memoryToIceAdapter, std::mutex& memoryMutex) :
        iceMemory(memoryToIceAdapter),
        memoryMutex(memoryMutex)
    {
        Logging::setTag("InstanceSegment");

        // oobbCache.setFetchFn([this](const ObjectID & id) -> std::optional<simox::OrientedBoxf>
        // {
        //     // Try to get OOBB from repository.
        //     if (std::optional<ObjectInfo> objectInfo = objectFinder.findObject(id))
        //     {
        //         try
        //         {
        //             return objectInfo->loadOOBB();
        //         }
        //         catch (const std::ios_base::failure& e)
        //         {
        //             // Give up - no OOBB information.
        //             ARMARX_WARNING << "Could not get OOBB of object " << id << ".\n- " << e.what();
        //             return std::nullopt;
        //         }
        //     }
        //     else
        //     {
        //         return std::nullopt;
        //     }
        // });

        // classNameToDatasetCache.setFetchFn([this](const std::string & className)
        // {
        //     std::optional<ObjectInfo> objectInfo = objectFinder.findObject(className);
        //     return objectInfo ? objectInfo->dataset() : "";
        // });
    }

    void Segment::defineProperties(armarx::PropertyDefinitionsPtr defs, const std::string& prefix)
    {
        defs->optional(p.coreSegmentName, prefix + "CoreSegmentName", "Name of the object instance core segment.");
        defs->optional(p.maxHistorySize, prefix + "MaxHistorySize", "Maximal size of object poses history (-1 for infinite).");
        defs->optional(p.discardSnapshotsWhileAttached, prefix + "DiscardSnapshotsWhileAttached",
                       "If true, no new snapshots are stored while an object is attached to a robot node.\n"
                       "If false, new snapshots are stored, but the attachment is kept in the new snapshots.");

        // decay.defineProperties(defs, prefix + "decay.");
    }

    void Segment::init()
    {
        ARMARX_CHECK_NOT_NULL(iceMemory.workingMemory);

        coreSegment = &iceMemory.workingMemory->addCoreSegment(p.coreSegmentName, arondto::ObjectInstance::toInitialAronType());
        coreSegment->setMaxHistorySize(p.maxHistorySize);
    }


    Segment::CommitStats Segment::commitArticulatedObjects(
        const std::string& providerName,
        const ::armarx::articulated_object::data::ArticulatedObjects& objects)
    {
        Segment::CommitStats stats;

        ::armarx::articulated_object::ArticulatedObjects boObjects;
        fromIce(objects, boObjects);

        commitArticulatedObjects(providerName, boObjects);

        return stats;
    }

    void Segment::commitArticulatedObjects(
        const std::string& providerName, 
        const ::armarx::articulated_object::ArticulatedObjects& objects)
    {
        ARMARX_CHECK_NOT_NULL(coreSegment);

        // Update memory.
        const MemoryID providerSegmentID = coreSegment->id().withProviderSegmentName(providerName);
        if (!coreSegment->hasProviderSegment(providerSegmentID.providerSegmentName))
        {
            coreSegment->addProviderSegment(providerSegmentID.providerSegmentName);
        }

        Commit commit;
        for (const Robot &object : objects)
            {
                EntityUpdate &update = commit.updates.emplace_back();
                {
                    update.entityID    = providerSegmentID.withEntityName(object.description.str());
                    update.timeArrived = TimeUtil::GetTime();
                    update.timeCreated = object.timestamp;

                    armarx::arondto::Robot dto;
                    toAron(dto, object);
                    // Search for object class.

                    update.instancesData.push_back(dto.toAron());
                }
            }

        // TODO check if this should be inside loop
        iceMemory.commit(commit);
    }



    // Segment::CommitStats Segment::commitObjectPoses(
    //     const std::string& providerName,
    //     const objpose::data::ProvidedObjectPoseSeq& providedPoses,
    //     std::optional<armem::Time> discardUpdatesUntil)
    // {
    //     CommitStats stats;

    //     // Build new poses.
    //     objpose::ObjectPoseSeq newObjectPoses;
    //     stats.numUpdated = 0;
    //     for (const objpose::data::ProvidedObjectPose& provided : providedPoses)
    //     {
    //         const IceUtil::Time timestamp = IceUtil::Time::microSeconds(provided.timestampMicroSeconds);

    //         // Check whether we have an old snapshot for this object.
    //         std::optional<objpose::ObjectPose> previousPose;
    //         const armem::wm::Entity* entity = findObjectEntity(armarx::fromIce(provided.objectID), providerName);
    //         if (entity)
    //         {
    //             const arondto::ObjectInstance data = getLatestInstanceData(*entity);

    //             previousPose = objpose::ObjectPose();
    //             fromAron(data, *previousPose);
    //         }

    //         bool discard = false;
    //         if (discardUpdatesUntil && timestamp < discardUpdatesUntil.value())
    //         {
    //             // Dicard updates temporarily (e.g. due to head movement).
    //             discard = true;
    //         }
    //         else if (previousPose)
    //         {
    //             if (p.discardSnapshotsWhileAttached && previousPose->attachment)
    //             {
    //                 // Discard update due to active attachemnt.
    //                 discard = true;
    //             }
    //             else if (timestamp == previousPose->timestamp)
    //             {
    //                 // Discard update as it is not new.
    //                 discard = true;
    //             }
    //         }

    //         if (!discard)
    //         {
    //             // Update the entity.
    //             stats.numUpdated++;

    //             objpose::ObjectPose& newPose = newObjectPoses.emplace_back();
    //             newPose.fromProvidedPose(provided, robot);

    //             if (previousPose && previousPose->attachment)
    //             {
    //                 // Keep current attachment.
    //                 ARMARX_CHECK(!p.discardSnapshotsWhileAttached);
    //                 newPose.attachment = previousPose->attachment;
    //             }

    //             if (newPose.objectID.dataset().empty())
    //             {
    //                 // Try to find the data set.
    //                 const std::string dataset = classNameToDatasetCache.get(newPose.objectID.className());
    //                 if (!dataset.empty())
    //                 {
    //                     newPose.objectID = { dataset, newPose.objectID.className(), newPose.objectID.instanceName() };
    //                 }
    //             }
    //             if (!provided.localOOBB)
    //             {
    //                 // Try to load oobb from disk.
    //                 newPose.localOOBB = getObjectOOBB(newPose.objectID);
    //             }
    //         }
    //     }

    //     commitObjectPoses(providerName, newObjectPoses);

    //     return stats;
    // }

    // void Segment::commitObjectPoses(const std::string& providerName, const ObjectPoseSeq& objectPoses)
    // {
    //     ARMARX_CHECK_NOT_NULL(coreSegment);

    //     // Update memory.
    //     const MemoryID providerSegmentID = coreSegment->id().withProviderSegmentName(providerName);
    //     if (!coreSegment->hasProviderSegment(providerSegmentID.providerSegmentName))
    //     {
    //         coreSegment->addProviderSegment(providerSegmentID.providerSegmentName);
    //     }

    //     Commit commit;
    //     for (const objpose::ObjectPose& pose : objectPoses)
    //     {
    //         EntityUpdate& update = commit.updates.emplace_back();
    //         {
    //             update.entityID = providerSegmentID.withEntityName(pose.objectID.str());
    //             update.timeArrived = TimeUtil::GetTime();
    //             update.timeCreated = pose.timestamp;
    //             update.confidence = pose.confidence;

    //             arondto::ObjectInstance dto;
    //             toAron(dto, pose);
    //             // Search for object class.
    //             if (auto instance = findClassInstance(pose.objectID))
    //             {
    //                 toAron(dto.classID, instance->id());
    //             }
    //             update.instancesData.push_back(dto.toAron());
    //         }

    //         iceMemory.commit(commit);
    //     }
    // }


    wm::CoreSegment& Segment::getCoreSegment()
    {
        ARMARX_CHECK_NOT_NULL(coreSegment);
        return *coreSegment;
    }

    const wm::CoreSegment& Segment::getCoreSegment() const
    {
        ARMARX_CHECK_NOT_NULL(coreSegment);
        return *coreSegment;
    }

    // objpose::ObjectPoseSeq Segment::getObjectPoses(IceUtil::Time now)
    // {
    //     ObjectPoseSeq objectPoses = getLatestObjectPoses();
    //     updateObjectPoses(objectPoses, now);
    //     return filterObjectPoses(objectPoses);
    // }


    // objpose::ObjectPoseSeq Segment::getObjectPosesByProvider(
    //     const std::string& providerName,
    //     IceUtil::Time now)
    // {
    //     ARMARX_CHECK_NOT_NULL(coreSegment);
    //     ObjectPoseSeq objectPoses = getLatestObjectPoses(coreSegment->getProviderSegment(providerName));
    //     updateObjectPoses(objectPoses, now);
    //     return filterObjectPoses(objectPoses);
    // }

    // armem::wm::Entity* Segment::findObjectEntity(const ObjectID& objectID, const std::string& providerName)
    // {
    //     ARMARX_CHECK_NOT_NULL(coreSegment);
    //     armem::MemoryID entityID = armem::MemoryID().withEntityName(objectID.str());
    //     if (providerName.empty())
    //     {
    //         for (auto& [_, prov] : *coreSegment)
    //         {
    //             if (prov.hasEntity(entityID.entityName))
    //             {
    //                 return &prov.getEntity(entityID);
    //             }
    //         }
    //         return nullptr;
    //     }
    //     else
    //     {
    //         entityID.providerSegmentName = providerName;
    //         if (coreSegment->hasProviderSegment(providerName))
    //         {
    //             armem::wm::ProviderSegment& prov = coreSegment->getProviderSegment(providerName);
    //             return prov.hasEntity(entityID.entityName) ? &prov.getEntity(entityID) : nullptr;
    //         }
    //         else
    //         {
    //             return nullptr;
    //         }
    //     }
    // }


    // void Segment::updateObjectPoses(ObjectPoseSeq& objectPoses, IceUtil::Time now)
    // {
    //     bool agentSynchronized = false;

    //     for (ObjectPose& objectPose : objectPoses)
    //     {
    //         updateObjectPose(objectPose, now, robot, agentSynchronized);
    //     }
    // }


    // void Segment::updateObjectPoses(
    //     ObjectPoseSeq& objectPoses,
    //     IceUtil::Time now,
    //     VirtualRobot::RobotPtr agent,
    //     bool& agentSynchronized) const
    // {
    //     for (ObjectPose& pose : objectPoses)
    //     {
    //         updateObjectPose(pose, now, agent, agentSynchronized);
    //     }
    // }


    // void Segment::updateObjectPose(
    //     ObjectPose& objectPose,
    //     IceUtil::Time now,
    //     VirtualRobot::RobotPtr agent,
    //     bool& agentSynchronized) const
    // {
    //     updateAttachement(objectPose, agent, agentSynchronized);

    //     if (decay.enabled)
    //     {
    //         decay.updateConfidence(objectPose, now);
    //     }
    // }


    // objpose::ObjectPoseSeq Segment::filterObjectPoses(const ObjectPoseSeq& objectPoses) const
    // {
    //     ObjectPoseSeq result;
    //     for (const ObjectPose& objectPose : objectPoses)
    //     {
    //         if (!(decay.enabled && objectPose.confidence < decay.removeObjectsBelowConfidence))
    //         {
    //             result.push_back(objectPose);
    //         }
    //     }
    //     return result;
    // }



    ::armarx::articulated_object::provider::ProviderInfo Segment::getProviderInfo(const std::string& providerName)
    {
        try
        {
            return providers.at(providerName);
        }
        catch (const std::out_of_range&)
        {
            std::stringstream ss;
            ss << "No provider with name '" << providerName << "' available.\n";
            ss << "Available are:\n";
            for (const auto& [name, _] : providers)
            {
                ss << "- '" << name << "'\n";
            }
            throw std::out_of_range(ss.str());
        }
    }


    std::optional<wm::EntityInstance> Segment::findClassInstance(const ObjectID& objectID)
    {
        const ObjectID classID = { objectID.dataset(), objectID.className() };
        try
        {
            for (const auto& [_, provSeg] : iceMemory.workingMemory->getCoreSegment("Class"))
            {
                return provSeg.getEntity(classID.str()).getLatestSnapshot().getInstance(0);
            }
            return std::nullopt;
        }
        catch (const armem::error::ArMemError&)
        {
            // Some segment or entity did not exist.
            return std::nullopt;
        }
    }



    void Segment::RemoteGui::setup(const Segment& data)
    {
        using namespace armarx::RemoteGui::Client;

        maxHistorySize.setValue(std::max(1, int(data.p.maxHistorySize)));
        maxHistorySize.setRange(1, 1e6);
        infiniteHistory.setValue(data.p.maxHistorySize == -1);
        discardSnapshotsWhileAttached.setValue(data.p.discardSnapshotsWhileAttached);

        GridLayout grid;
        int row = 0;
        grid.add(Label("Max History Size"), {row, 0}).add(maxHistorySize, {row, 1});
        row++;
        grid.add(Label("Infinite History Size"), {row, 0}).add(infiniteHistory, {row, 1});
        row++;
        grid.add(Label("Discard Snapshots while Attached"), {row, 0}).add(discardSnapshotsWhileAttached, {row, 1});
        row++;

        group.setLabel("Data");
        group.addChild(grid);
    }

    void Segment::RemoteGui::update(Segment& data)
    {
        if (infiniteHistory.hasValueChanged() || maxHistorySize.hasValueChanged()
            || discardSnapshotsWhileAttached.hasValueChanged())
        {
            std::scoped_lock lock(data.memoryMutex);

            if (infiniteHistory.hasValueChanged() || maxHistorySize.hasValueChanged())
            {
                data.p.maxHistorySize = infiniteHistory.getValue() ? -1 : maxHistorySize.getValue();
                if (data.coreSegment)
                {
                    data.coreSegment->setMaxHistorySize(long(data.p.maxHistorySize));
                }
            }

            data.p.discardSnapshotsWhileAttached = discardSnapshotsWhileAttached.getValue();
        }
    }

}
