#pragma once

#include "ArmarXCore/core/application/properties/PropertyDefinitionContainer.h"
#include "ArmarXGui/libraries/RemoteGui/Client/Widgets.h"
#include <RobotAPI/interface/core/articulated_object/provider.h>
#include <map>
#include <string>
#include <optional>
#include <mutex>

#include <SimoxUtility/caching/CacheMap.h>
#include <SimoxUtility/shapes/OrientedBox.h>

#include <ArmarXCore/core/logging/Logging.h>

#include <RobotAPI/interface/core/RobotState.h>

#include <RobotAPI/libraries/ArmarXObjects/ObjectID.h>
#include <RobotAPI/libraries/ArmarXObjects/ObjectPose.h>
#include <RobotAPI/libraries/ArmarXObjects/ObjectFinder.h>

#include <RobotAPI/libraries/armem/core/workingmemory/Memory.h>
#include <RobotAPI/libraries/armem/server/MemoryToIceAdapter.h>
#include <RobotAPI/libraries/armem_objects/aron/ObjectInstance.aron.generated.h>

#include <RobotAPI/libraries/aron/common/aron/new_types.h>
#include <RobotAPI/interface/core/articulated_object/types.h>

namespace armarx::armem::server::obj::articulated_object
{

    class Segment : public armarx::Logging
    {
    public:

        struct CommitStats
        {
            int numUpdated = 0;
        };
        using ObjectPose = objpose::ObjectPose;
        using ObjectPoseSeq = objpose::ObjectPoseSeq;


    public:

        Segment(server::MemoryToIceAdapter& iceMemory,
                std::mutex& memoryMutex);


        void defineProperties(armarx::PropertyDefinitionsPtr defs, const std::string& prefix = "");

        void init();


        CommitStats commitArticulatedObjects(
        const std::string& providerName,
        const ::armarx::articulated_object::data::ArticulatedObjects& objects);

        void commitArticulatedObjects(
            const std::string& providerName,
            const ::armarx::articulated_object::ArticulatedObjects& objects
        );

        wm::CoreSegment& getCoreSegment();
        const wm::CoreSegment& getCoreSegment() const;


        // objpose::ObjectPoseSeq getObjectPoses(IceUtil::Time now);
        // objpose::ObjectPoseSeq getObjectPosesByProvider(const std::string& providerName, IceUtil::Time now);

        // wm::Entity* findObjectEntity(const ObjectID& objectID, const std::string& providerName = "");
        // std::optional<simox::OrientedBoxf> getObjectOOBB(const ObjectID& id);

        ::armarx::articulated_object::provider::ProviderInfo getProviderInfo(const std::string& providerName);

       
        // static ObjectPoseSeq getLatestObjectPoses(const wm::CoreSegment& coreSeg);
        // static ObjectPoseSeq getLatestObjectPoses(const wm::ProviderSegment& provSeg);
        // static ObjectPose getLatestObjectPose(const wm::Entity& entity);

        // static void getLatestObjectPoses(const wm::CoreSegment& coreSeg, ObjectPoseSeq& out);
        // static void getLatestObjectPoses(const wm::ProviderSegment& provSeg, ObjectPoseSeq& out);
        // static void getLatestObjectPose(const wm::Entity& entity, ObjectPose& out);

        // static arondto::ObjectInstance getLatestInstanceData(const wm::Entity& entity);


    private:

        ObjectPoseSeq getLatestObjectPoses() const;

        void updateObjectPoses(
            ObjectPoseSeq& objectPoses,
            IceUtil::Time now);
        void updateObjectPoses(
            ObjectPoseSeq& objectPoses,
            IceUtil::Time now,
            VirtualRobot::RobotPtr agent,
            bool& agentSynchronized
        ) const;
        void updateObjectPose(
            ObjectPose& objectPose,
            IceUtil::Time now,
            VirtualRobot::RobotPtr agent,
            bool& agentSynchronized
        ) const;


        ObjectPoseSeq filterObjectPoses(const ObjectPoseSeq& objectPoses) const;


        void storeDetachedSnapshot(
            wm::Entity& entity,
            const arondto::ObjectInstance& data,
            Time now,
            bool commitAttachedPose);


        std::optional<wm::EntityInstance> findClassInstance(const ObjectID& objectID);


        friend struct DetachVisitor;


    public:

        RobotStateComponentInterfacePrx robotStateComponent;
        VirtualRobot::RobotPtr robot;

        using ProviderMap = std::map<std::string, ::armarx::articulated_object::provider::ProviderInfo>;

        ProviderMap providers;


        // ObjectFinder objectFinder;

        /// Decay model.
        // Decay decay;


    private:

        server::MemoryToIceAdapter& iceMemory;
        wm::CoreSegment* coreSegment = nullptr;
        std::mutex& memoryMutex;


        struct Properties
        {
            std::string coreSegmentName = "ArticulatedObjectInstance";
            long maxHistorySize = -1;
            bool discardSnapshotsWhileAttached = true;
        };
        Properties p;


        /// Caches results of attempts to retrieve the OOBB from ArmarXObjects.
        simox::caching::CacheMap<ObjectID, std::optional<simox::OrientedBoxf>> oobbCache;

        /// Class name -> dataset name.
        simox::caching::CacheMap<std::string, std::string> classNameToDatasetCache;


    public:

        struct RemoteGui
        {
            armarx::RemoteGui::Client::GroupBox group;

            armarx::RemoteGui::Client::IntSpinBox maxHistorySize;
            armarx::RemoteGui::Client::CheckBox infiniteHistory;
            armarx::RemoteGui::Client::CheckBox discardSnapshotsWhileAttached;

            void setup(const Segment& data);
            void update(Segment& data);
        };

    };

}  // namespace armarx::armem::server::obj::articulated_object
