/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::armem_objects::Adapter
 * @author     Rainer Kartmann ( rainer dot kartmann at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <RobotAPI/interface/core/articulated_object/types.h>
#include <mutex>

#include <VirtualRobot/VirtualRobot.h>

#include <ArmarXGui/libraries/RemoteGui/Client/Widgets.h>

#include <RobotAPI/interface/armem/server/ObjectMemoryInterface.h>
#include <RobotAPI/interface/core/RobotState.h>

#include <RobotAPI/components/ArViz/Client/Client.h>

#include <RobotAPI/libraries/armem/server/MemoryToIceAdapter.h>
#include <RobotAPI/libraries/armem_objects/server/articulated_object/Segment.h>
// #include <RobotAPI/libraries/armem_objects/server/articulated_object/Decay.h>
// #include <RobotAPI/libraries/armem_objects/server/articulated_object/Visu.h>
// #include <RobotAPI/libraries/armem_objects/server/articulated_object/RobotHeadMovement.h>


#define ICE_CURRENT_ARG const Ice::Current& = Ice::emptyCurrent


namespace armarx::armem::server::obj::articulated_object
{

    /**
     * @brief Helps implementing the `armarx::armem::server::ObjectInstanceSegmentInterface`.
     */
    class SegmentAdapter :
        virtual public armarx::Logging
        // , virtual public armarx::armem::server::ArticulatedObjectInstanceSegment
    {
    public:

        SegmentAdapter(MemoryToIceAdapter& iceMemory, std::mutex& memoryMutex);

        std::string getName() const;
        void defineProperties(armarx::PropertyDefinitionsPtr defs, const std::string& prefix = "");

        void init();
        void connect(
            viz::Client arviz,
            DebugObserverInterfacePrx debugObserver
        );

        // ARTICULATED OBJECTS
        void reportArticulatedObjects(const std::string& providerName, const ::armarx::articulated_object::data::ArticulatedObjects& objects, ICE_CURRENT_ARG) override;

    private:

        void updateProviderInfo(const std::string& providerName, const objpose::ProviderInfo& info);

        void updateObjectPoses(const std::string& providerName, const objpose::data::ProvidedObjectPoseSeq& providedPoses);
        void handleProviderUpdate(const std::string& providerName);


        void updateArticulatedObjects(const std::string& providerName, const ::armarx::articulated_object::data::ArticulatedObjects& objects);

        // Visualization

        void visualizeRun();


    private:

        viz::Client arviz;
        DebugObserverInterfacePrx debugObserver;

        articulated_object::Segment segment;
        std::mutex& memoryMutex;

        // articulated_object::Visu visu;
        // std::mutex visuMutex;

    public:

        struct RemoteGui
        {
            armarx::RemoteGui::Client::GroupBox group;
            armarx::RemoteGui::Client::VBoxLayout layout;

            // articulated_object::Visu::RemoteGui visu;
            articulated_object::Segment::RemoteGui segment;

            void setup(const SegmentAdapter& adapter);
            void update(SegmentAdapter& adapter);
        };

    };

}

#undef ICE_CURRENT_ARG
