#pragma once


#include <ArmarXCore/core/logging/Logging.h>
#include <ArmarXCore/core/services/tasks/TaskUtil.h>

#include <ArmarXGui/libraries/RemoteGui/Client/Widgets.h>

#include <RobotAPI/components/ArViz/Client/Client.h>
#include <RobotAPI/libraries/ArmarXObjects/ObjectPose.h>


namespace armarx
{
    class ObjectFinder;
}
namespace armarx::armem::server::obj::instance
{

    /**
     * @brief Models decay of object localizations by decreasing the confidence
     * the longer the object was not localized.
     */
    class Visu : public armarx::Logging
    {
    public:

        void defineProperties(armarx::PropertyDefinitionsPtr defs, const std::string& prefix = "visu.");

        std::vector<viz::Layer> visualizeCommit(
            const std::map<std::string, objpose::ObjectPoseSeq>& objectPoses,
            const ObjectFinder& objectFinder
        ) const;

        /// Visualize the given object poses, with one layer per provider.
        std::vector<viz::Layer> visualizeCommit(
            const objpose::ObjectPoseSeq& objectPoses,
            const ObjectFinder& objectFinder
        ) const;

        viz::Layer visualizeProvider(
            const std::string& providerName,
            const objpose::ObjectPoseSeq& objectPoses,
            const ObjectFinder& objectFinder
        ) const;


        void visualizeObjectPose(
            viz::Layer& layer,
            const objpose::ObjectPose& objectPose,
            const ObjectFinder& objectFinder
        ) const;


    public:

        viz::Client arviz;

        bool enabled = true;
        float frequencyHz = 25;

        bool inGlobalFrame = true;
        float minConfidence = -1;
        float alpha = 1.0;
        bool alphaByConfidence = false;
        bool oobbs = false;
        bool objectFrames = false;
        float objectFramesScale = 1.0;

        SimpleRunningTask<>::pointer_type updateTask;


        struct RemoteGui
        {
            armarx::RemoteGui::Client::GroupBox group;

            armarx::RemoteGui::Client::CheckBox enabled;

            armarx::RemoteGui::Client::CheckBox inGlobalFrame;
            armarx::RemoteGui::Client::FloatSlider alpha;
            armarx::RemoteGui::Client::CheckBox alphaByConfidence;
            armarx::RemoteGui::Client::CheckBox oobbs;
            armarx::RemoteGui::Client::CheckBox objectFrames;
            armarx::RemoteGui::Client::FloatSpinBox objectFramesScale;

            void setup(const Visu& visu);
            void update(Visu& visu);
        };

    };

}
