/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <stack>

// BaseClass
#include <RobotAPI/libraries/aron/core/io/dataIO/Reader.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/navigator/data/Navigator.h>
#include <RobotAPI/libraries/aron/core/io/dataIO/reader/navigator/NavigatorReaderToken.h>


namespace armarx::aron::dataIO::reader
{
    class NavigatorReader;
    typedef std::shared_ptr<NavigatorReader> NavigatorReaderPtr;

    class NavigatorReader :
        virtual public ReaderInterface
    {
    public:
        using PointerType = NavigatorReaderPtr;

        // constructors
        NavigatorReader() = delete;
        NavigatorReader(const datanavigator::NavigatorPtr& n);
        NavigatorReader(const data::AronDataPtr& n);

        virtual int readStartDict() override;
        virtual bool readEndDict() override;
        virtual int readStartList() override;
        virtual bool readEndList() override;

        virtual std::tuple<std::vector<int>, std::string> readStartNDArray() override;
        virtual bool readEndNDArray(unsigned char*) override;

        virtual void readPrimitive(int&) override;
        virtual void readPrimitive(long&) override;
        virtual void readPrimitive(float&) override;
        virtual void readPrimitive(double&) override;
        virtual void readPrimitive(std::string&) override;
        virtual void readPrimitive(bool&) override;

        virtual std::string readKey() override;
        virtual void loadMember(const std::string&) override;

        virtual data::Descriptor getTypeOfNext(const type::Descriptor hint = type::Descriptor::eUnknown) const override;

    private:
        datanavigator::NavigatorPtr getNext();
        datanavigator::NavigatorPtr getNextAndIncrease();

    private:
        datanavigator::NavigatorPtr input;
        bool readInitialStart = false;
        std::stack<NavigatorReaderTokenPtr> stack;
    };
}
