/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "Navigator.h"

// ArmarX
#include <RobotAPI/libraries/aron/core/Exception.h>
#include <RobotAPI/libraries/aron/core/navigator/data/NavigatorFactory.h>

namespace armarx::aron::datanavigator
{
    // static data members
    const NavigatorFactoryPtr Navigator::FACTORY = NavigatorFactoryPtr(new NavigatorFactory());

    // constructors
    Navigator::Navigator(const data::Descriptor& descriptor, const Path& path) :
        aron::Navigator<data::Descriptor, data::AronData>::Navigator(descriptor, path)
    {
    }

    // static methods
    NavigatorPtr Navigator::FromAronData(const data::AronDataPtr& a, const Path& path)
    {
        return FACTORY->create(a, path);
    }

    std::vector<NavigatorPtr> Navigator::FromAronData(const std::vector<data::AronDataPtr>& a, const Path& path)
    {
        std::vector<NavigatorPtr> ret;
        for (const auto& aron : a)
        {
            ret.push_back(Navigator::FromAronData(aron, path));
        }
        return ret;
    }
    std::vector<data::AronDataPtr> Navigator::ToAronData(const std::vector<NavigatorPtr>& a)
    {
        std::vector<data::AronDataPtr> ret;
        for (const auto& aron : a)
        {
            ret.push_back(aron->getResult());
        }
        return ret;
    }

    void Navigator::CheckDataNavigatorPtrForNull(const std::string& c, const std::string& m, const NavigatorPtr& data)
    {
        if (data.get() == nullptr)
        {
            throw error::AronException(c, m, "Could not make use of an NavigatorPtr. The Ptr was NULL");
        }
    }

    void Navigator::CheckDataNavigatorPtrForNull(const std::string& c, const std::string& m, const Path& p, const NavigatorPtr& data)
    {
        if (data.get() == nullptr)
        {
            throw error::AronException(c, m, "Could not make use of an NavigatorPtr. The Ptr was NULL", p);
        }
    }

    NavigatorPtr Navigator::navigateAbsolute(const Path& path) const
    {
        throw error::AronException("Navigator", "navigateAbsolute", "Could not navigate through a non container navigator. The input path was: " + path.toString(), getPath());
    }

    NavigatorPtr Navigator::navigateRelative(const Path& path) const
    {
        throw error::AronException("Navigator", "navigateRelative", "Could not navigate through a non container navigator. The input path was: " + path.toString(), getPath());
    }


    /*AronContainerDataNavigatorPtr AronContainerDataNavigator::DynamicCast(const NavigatorPtr& n)
    {
        AronContainerDataNavigatorPtr casted = std::dynamic_pointer_cast<AronContainerDataNavigator>(n);
        return casted;
    }

    AronContainerDataNavigatorPtr AronContainerDataNavigator::DynamicCastAndCheck(const NavigatorPtr& n)
    {
        CheckDataNavigatorPtrForNull("AronContainerDataNavigator", "DynamicCastAndCheck[Before]", n);
        AronContainerDataNavigatorPtr casted = AronContainerDataNavigator::DynamicCast(n);
        CheckDataNavigatorPtrForNull("AronContainerDataNavigator", "DynamicCastAndCheck[After]", n->getPath(), casted);
        return casted;
    }

    NavigatorPtr AronContainerDataNavigator::navigateRelative(const Path& path) const
    {
        Path absoluteFromHere = path.getWithoutPrefix(getPath());
        return navigateAbsolute(absoluteFromHere);
    }*/
}
