/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "Primitive.h"

#include <RobotAPI/libraries/aron/core/navigator/data/NavigatorFactory.h>
#include <RobotAPI/libraries/aron/core/navigator/type/primitive/Primitive.h>

namespace armarx::aron::datanavigator
{
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    /* constructors */ \
    upperType##Navigator::upperType##Navigator(const data::Aron##upperType##Ptr& o, const Path& path) : \
        aron::Navigator<data::Descriptor, data::AronData>::Navigator(data::Descriptor::e##upperType, path), \
        Navigator(data::Descriptor::e##upperType, path), \
        aron(o) \
    { \
    } \
    \
    upperType##Navigator::upperType##Navigator(const Path& path) : \
        aron::Navigator<data::Descriptor, data::AronData>::Navigator(data::Descriptor::e##upperType, path), \
        Navigator(data::Descriptor::e##upperType, path), \
        aron(new data::Aron##upperType()) \
    { \
    } \
    \
    upperType##Navigator::upperType##Navigator(const lowerType& d, const Path& path) : \
        upperType##Navigator(data::Aron##upperType##Ptr(new data::Aron##upperType(d)), path) \
    { \
    } \
    \
    /* operators */ \
    upperType##Navigator::operator lowerType() const \
    { \
        return aron->value; \
    } \
    \
    bool upperType##Navigator::operator==(const upperType##Navigator& other) const \
    { \
        const auto& otherAron = other.toAron##upperType##Ptr(); \
        if(aron->value != otherAron->value) \
            return false; \
        return true; \
    } \
    \
    bool upperType##Navigator::equalsDataNavigator(const NavigatorPtr& other) const \
    { \
        if(other == nullptr) \
        { \
            return false; \
        } \
        upperType##NavigatorPtr casted = DynamicCast(other); \
        return equalsDeep(casted); \
    } \
    \
    bool upperType##Navigator::equalsDeep(const upperType##NavigatorPtr& other) const \
    { \
        /* only for consistency. There is no "deep" comparison. */ \
        if (other == nullptr) \
        { \
            return false; \
        } \
        return *this == *other; \
    } \
    \
    /* static methods */ \
    upperType##NavigatorPtr upperType##Navigator::DynamicCast(const NavigatorPtr& n) \
    { \
        upperType##NavigatorPtr casted = std::dynamic_pointer_cast<upperType##Navigator>(n); \
        return casted; \
    } \
    \
    upperType##Navigator upperType##Navigator::DynamicCast(Navigator& n) \
    { \
        return dynamic_cast<upperType##Navigator&>(n); \
    } \
    \
    upperType##NavigatorPtr upperType##Navigator::DynamicCastAndCheck(const NavigatorPtr& n) \
    { \
        CheckDataNavigatorPtrForNull("Aron"+std::string(#upperType)+"DataNavigator", "DynamicCastAndCheck[Before]", n); \
        upperType##NavigatorPtr casted = upperType##Navigator::DynamicCast(n); \
        CheckDataNavigatorPtrForNull("Aron"+std::string(#upperType)+"DataNavigator", "DynamicCastAndCheck[After]", n->getPath(), casted); \
        return casted; \
    } \
    \
    upperType##NavigatorPtr upperType##Navigator::FromAron##upperType##Ptr(const data::Aron##upperType##Ptr& aron) \
    { \
        return std::make_shared<upperType##Navigator>(aron); \
    } \
    \
    data::Aron##upperType##Ptr upperType##Navigator::ToAron##upperType##Ptr(const upperType##NavigatorPtr& navigator) \
    { \
        return navigator ? navigator->toAron##upperType##Ptr() : nullptr; \
    } \
    \
    /* public member functions */ \
    void upperType##Navigator::setValue(const lowerType& x) \
    { \
        aron->value = x; \
    } \
    \
    lowerType upperType##Navigator::getValue() const \
    { \
        return aron->value; \
    } \
    \
    data::Aron##upperType##Ptr upperType##Navigator::toAron##upperType##Ptr() const \
    { \
        return aron; \
    } \
    \
    /* virtual implementations */ \
    data::AronDataPtr upperType##Navigator::getResult() const \
    { \
        return aron; \
    } \
    \
    std::string upperType##Navigator::getName() const \
    { \
        return "Aron" + std::string(#upperType); \
    } \
    \
    bool upperType##Navigator::fullfillsType(const typenavigator::NavigatorPtr& type) const \
    { \
        return Resolver::Correspond(type->getDescriptor(), getDescriptor()); \
    } \
    \
    typenavigator::NavigatorPtr upperType##Navigator::recalculateType() const \
    { \
        return typenavigator::upperType##NavigatorPtr(new typenavigator::upperType##Navigator(getPath())); \
    } \
    \
    std::vector<NavigatorPtr> upperType##Navigator::getChildren() const \
    { \
        return {}; \
    } \
    \
    size_t upperType##Navigator::childrenSize() const \
    { \
        return 0; \
    } \

    HANDLE_PRIMITIVE_DATA
#undef RUN_ARON_MACRO
}
